# =========================================================================
# File: __FILENAME__
#
# Copyright (c) __YEAR__ and onwards, __AUTHOR__ <__AUTHOR_EMAIL__>
#
# LICENCE:
#
#   This file is distributed under the terms of the __LICENCE_NAME__.
#   See the __LICENCE_FILE__ file, which should have been distributed with
#   this file, for details. If you did not receive the __LICENCE_FILE__ file,
#   see:
#
#   __LICENCE_URI__
#
# DESCRIPTION:
#
#   Makefile for __PROJECT__
#
# USAGE:
#
#   make <target>
#
#   Run 'make help' to see a list of all targets and what they do
#
# EXAMPLES:
#
#   make && sudo make install
#   make doc
#
# DEPENDENCIES:
#
#   awk (tested with GNU awk)
#   C compiler (tested with GNU gcc)
#   Doxygen (for targets: doc_doxygen Doxyfile)
#   Gentoo Portage (for targets: dist_ebuild)
#   grep (tested with GNU grep)
#   make (tested with GNU make)
#   NaturalDocs (for targets: doc_naturaldocs)
#   rsync (for targets: dist doc_api_push)
#   SSH (for targets: dist doc_api_push)
#   tar (tested with GNU tar; for targets: dist)
#
# TODO:
# 
#   - Nothing, this code is perfect
#  
# MODIFICATIONS:
#
#   __AUTHOR__ <__AUTHOR_EMAIL__> (__DATE__): Initial revision
# =========================================================================

## (Unix) Name of the project
NAME = __PROJECT__

## Version of the project
VERSION = `cat VERSION`

## Programs used by Makefile
#--------------------------------------------------------------------------

# If these are not in your $PATH, specify the full path
AWK         = gawk
CC          = gcc
DOXYGEN     = doxygen
EBUILD      = ebuild
GREP        = grep
NATURALDOCS = NaturalDocs
RSYNC       = rsync
SCP	    = scp
SSH	    = ssh
TAR         = tar

#--------------------------------------------------------------------------

## Compilation-related
#--------------------------------------------------------------------------

## Flags for the C compiler
CFLAGS = -I. -Wall

## The base directory for installation (prepended *before* $(PREFIX)
DESTDIR =

## Installation prefix (e.g. 'make "PREFIX = /usr" install' would install
## files in /usr/lib, /usr/include, and /usr/man)
PREFIX  = /usr/local

## C source files
CFILES  = ${NAME}.c

## C header files
HFILES  = ${NAME}.h

## Miscellaneous files
MFILES  = valgrind-log*

## Object files
OFILES  = ${NAME}.o

## Shared object (library) files
SOFILES = lib${NAME}.so

## Test program's C source files
TEST_CFILES  = test-${NAME}.c

## Test program's object files
TEST_OFILES  = test-${NAME}.o

## Test program's binary file
TEST_BINARY  = test-${NAME}

# Path to valgrind binary
VALGRIND_BIN  = `which valgrind`
VALGRIND_OPTS = -v --leak-check=yes --leak-resolution=high --show-reachable=yes

#--------------------------------------------------------------------------

## Distribution-related
#--------------------------------------------------------------------------

## Write distribution files here
DISTDIR = /tmp

## Use this directory to generate distribution files
TMPDIR  = /tmp

## Name of the tar'd, bzip2'd distfile
DIST_TBZ = $(NAME)-$(VERSION).tbz2

## Name of the tar'd, gzip'd distfile
DIST_TGZ = $(NAME)-$(VERSION).tgz

## Targets for package generation
DIST_PKG = 
#DIST_PKG = dist_bsdpkg dist_deb dist_ebuild dist_rpm dist_solpkg

## Set this to disable ebuild generation
DIST_EBUILD_DISABLED =

## Set this to disable Solaris package generation
DIST_SOLPKG_DISABLED =

## Portage overlay directory (Gentoo-specific)
PORTDIR_OVERLAY = `$(GREP) ^PORTDIR_OVERLAY /etc/make.conf 2>/dev/null | \
                   $(AWK) -F'=' '{ print $$2 }'`

## Portage category (Gentoo-specific)
PORTCAT = dev-libs

## Host on which to build Solaris 8 Intel packages (SSH-style user@host)
SOL8I_HOST = 

## Prefix for building Solaris 8 Intel packages
SOL8I_PREFIX = build/pkg

## Temporary directory for building Solaris 8 Intel packages
SOL8I_TMPDIR = /tmp

## Host on which to build Solaris 8 Sparc packages (SSH-style user@host)
SOL8S_HOST =

## Temporary directory for building Solaris 8 Sparc packages
SOL8S_TMPDIR = /tmp

## Prefix for building Solaris 8 Sparc packages
SOL8S_PREFIX = build/pkg

#--------------------------------------------------------------------------

## Documentation-related
#--------------------------------------------------------------------------

# Doxygen config
DOXY_OUTPUT_DIRECTORY = docs
DOXY_OPTIMIZE_OUTPUT  = FOR_C
DOXY_EXTRACT_ALL      = YES
DOXY_RECURSIVE        = YES
DOXY_GENERATE_MAN     = YES
DOXY_GENERATE_LATEX   = NO

# NaturalDocs config
NDOCS_INPUT_DIRECTORY   = ./
NDOCS_OUTPUT_DIRECTORY  = docs
NDOCS_OUTPUT_FORMAT     = html
NDOCS_PROJECT_DIRECTORY = /tmp/ndproject/${NDOCS_PROJECT_NAME}

# API doc server config
API_DOC_PROGRAM = ${NATURALDOCS}
API_DOC_SERVER  = www.jmglov.net
API_DOC_LDIR    =
API_DOC_RDIR    = /var/www/localhost/htdocs/opensource/${NAME}/doc/api

# LXR config
LXR_SERVER       = jglover.tfcc.com
LXR_HTDOCS       = /var/www/localhost/htdocs
LXR_DIR          = lxr
LXR_NAME         = ${NAME}
LXR_PATH         = ${LXR_HTDOCS}/${LXR_DIR}/${LXR_NAME}
LXR_BINDIR       = ../bin
LXR_HTMLDIR      = .
LXR_LIBDIR       = ../lib
LXR_SOURCEROOT   = ./src
LXR_SRCROOTNAME  = ${NAME}
LXR_INCPREFIX    = ./include
LXR_DBDIR        = ./db
LXR_GLIMPSEBIN   = glimpse
LXR_GLIMPSEINDEX = ${LXR_GLIMPSEBIN}index
LXR_GENXREF      = genxref
LXR_BASEURL      = http://${LXR_SERVER}/${LXR_DIR}/${LXR_NAME}/

## Filename of the source manpage that we will install
MANFILE = $(DOXY_MAN)/man3/${NAME}.h.3

## Copy the source manpage to this file when installing
MANPAGE = $(MANDIR)/${NAME}.3

## Man page section directory
MANDIR  = man3

#--------------------------------------------------------------------------


## Build lib__PROJECT__ as a shared object
build : $(CFILES) $(HFILES)
	$(CC) $(CFLAGS) -shared -fPIC -o $(SOFILES) $(CFILES)

clean : 
	- rm -rf $(OFILES) $(SOFILES) $(TEST_OFILES) $(TEST_BINARY) $(MFILES)

debug :
	$(MAKE) "CFLAGS = $(CFLAGS) -g" build

dist : man
	@ if test -z "$(VERSION)"; then \
	  echo 'Version is borked!'; \
	fi
#	Run doxygen if it is available--might as well give the user some docs
	@ if type -p $(DOXYGEN); then\
	  echo $(DOXYGEN);\
	  $(DOXYGEN);\
	fi
	@ echo $(RSYNC) -aC --delete --exclude *.ebuild . \
	$(TMPDIR)/${NAME}-$(VERSION)
	@ $(RSYNC) -aC --delete --exclude *.ebuild . \
	$(TMPDIR)/${NAME}-$(VERSION)
	@ version=$(VERSION); cd $(TMPDIR);\
	echo $(TAR) cjf $(DISTDIR)/${NAME}-$$version.tbz2 ${NAME}-$$version;\
	echo $(TAR) czf $(DISTDIR)/${NAME}-$$version.tgz ${NAME}-$$version;\
	$(TAR) cjf $(DISTDIR)/${NAME}-$$version.tbz2 ${NAME}-$$version;\
	$(TAR) czf $(DISTDIR)/${NAME}-$$version.tgz ${NAME}-$$version
#	Generate packages
	@ for i in $(DIST_PKG); do\
	  if test "$$i" = "dist_solpkg"; then\
	    echo $(MAKE) "MAKE = gmake" "TAR = gtar" $$i;\
	    $(MAKE) "MAKE = gmake" "TAR = gtar" $$i;\
	  else\
	    echo $(MAKE) $$i;\
	    $(MAKE) $$i;\
	  fi;\
	done

## Builds a BSD package
dist_bsdpkg :
	@ echo 'BSD package generation currently not implemented'

## Builds a Debian package
dist_deb:
	@ echo 'Debian package generation currently not implemented'

## Builds a Gentoo ebuild
dist_ebuild :
	@ if test -z "$(DIST_EBUILD_DISABLED)"; then\
	  echo cp distfiles/${NAME}.ebuild.template \
	  distfiles/${NAME}-$(VERSION).ebuild;\
	  cp distfiles/${NAME}.ebuild.template distfiles/${NAME}-$(VERSION).ebuild;\
	  echo "Created ${NAME}-$(VERSION).ebuild";\
	  echo "You should verify that it is correct";\
	else\
	  echo 'DIST_EBUILD_DISABLED macro set, skipping ebuild generation';\
	fi

## Builds an RPM
dist_rpm :
	@ echo 'RPM generation currently not implemented'

## Builds a Solaris package
dist_solpkg :
	@ if test -z "$(DIST_SOLPKG_DISABLED)"; then\
	  echo;\
	  echo 'Building Solaris 8 Intel package:';\
	  echo $(SCP) $(DISTDIR)/$(DIST_TGZ) \
	  $(SOL8I_HOST):$(SOL8I_TMPDIR);\
	  $(SCP) $(DISTDIR)/$(DIST_TGZ) $(SOL8I_HOST):$(SOL8I_TMPDIR);\
	  echo $(SSH) $(SOL8I_HOST) \'cd $(SOL8I_TMPDIR)\;\\;\
	  echo ' '$(TAR) xzf $(DIST_TBZ)\;\\;\
	  echo ' 'cd $(NAME)-$(VERSION)\;\\;\
	  echo ' '$(MAKE) "PREFIX = $(SOL8I_PREFIX)" install\';\
	else\
	  echo 'DIST_SOLPKG_DISABLED macro set, skipping Solaris '\
	  'package generation';\
	fi

## Generate documentation
doc: clean
	@ if test -x "${API_DOC_PROGRAM}"; then ${MAKE} doc_api; else true; fi
	@ if test -n "${LXR_SERVER}"; then ${MAKE} doc_lxr; else true; fi

## Generate API documentation with Doxygen and push it out to a webserver
doc_api:
	@ echo "Generating documentation with ${API_DOC_PROGRAM}"
	@ if [ "${API_DOC_PROGRAM}" = "${DOXYGEN}" ]; then \
	  ${MAKE} doc_doxygen doc_api_push \
	    API_DOC_LDIR="${DOXY_OUTPUT_DIRECTORY}/html"; \
	  rm -rf ${DOXY_OUTPUT_DIRECTORY}; \
	fi
	@ if [ "${API_DOC_PROGRAM}" = "${NATURALDOCS}" ]; then \
	  ${MAKE} doc_naturaldocs doc_api_push \
	    API_DOC_LDIR="${NDOCS_OUTPUT_DIRECTORY}"; \
	  rm -rf ${NDOCS_OUTPUT_DIRECTORY} ${NDOCS_PROJECT_DIRECTORY}; \
	fi

## Generate Doxygen's config file
Doxyfile:
	${DOXYGEN} -g >/dev/null 2>&1
	cp Doxyfile Doxyfile.orig
	sed \
	  -e 's/^\(PROJECT_NAME \+\)=.\+$$/\1= ${DOXY_PROJECT_NAME}/' \
	  -e 's/^\(OUTPUT_DIRECTORY \+\)=.\+$$/\1= ${DOXY_OUTPUT_DIRECTORY}/' \
	  -e 's/^\(OPTIMIZE_OUTPUT_${DOXY_OPTIMIZE_OUTPUT} \+\)=.\+$$/\1= YES/' \
	  -e 's/^\(EXTRACT_ALL \+\)=.\+$$/\1= ${DOXY_EXTRACT_ALL}/' \
	  -e 's/^\(RECURSIVE \+\)=.\+$$/\1= ${DOXY_RECURSIVE}/' \
	  -e 's/^\(GENERATE_MAN \+\)=.\+$$/\1= ${DOXY_GENERATE_MAN}/' \
	  -e 's/^\(GENERATE_LATEX \+\)=.\+$$/\1= ${DOXY_GENERATE_LATEX}/' \
	  Doxyfile.orig >Doxyfile
	rm Doxyfile.orig

## Generate API documentation with Doxygen
doc_doxygen: Doxyfile
	@ ${DOXYGEN} >/dev/null 2>&1

## Generate API documentation with NaturalDocs
doc_naturaldocs:
	mkdir -p ${NDOCS_OUTPUT_DIRECTORY} ${NDOCS_PROJECT_DIRECTORY}
	@ ${NATURALDOCS} -i ${NDOCS_INPUT_DIRECTORY} \
	  -o ${NDOCS_OUTPUT_FORMAT} ${NDOCS_OUTPUT_DIRECTORY} \
	  -p ${NDOCS_PROJECT_DIRECTORY}
	rm -rf ${NDOCS_PROJECT_DIRECTORY}

## Push out API documentation to a webserver
doc_api_push:
	@ ${SSH} ${API_DOC_SERVER} test -d ${API_DOC_RDIR}; \
	if [ $$? -ne 0 ]; then \
	  ${SSH} ${API_DOC_SERVER} mkdir -p ${API_DOC_RDIR}; \
	fi
	${RSYNC} -av -e ssh --delete ${API_DOC_LDIR}/ \
	  ${API_DOC_SERVER}:${API_DOC_RDIR}/
	${SSH} ${API_DOC_SERVER} chmod -R 755 ${API_DOC_RDIR}

## Generate browsable source with LXR
doc_lxr: lxr.conf
	@ if ! ssh ${LXR_SERVER} test -d ${LXR_PATH}; then \
	  if ssh ${LXR_SERVER} type -a ${WEBAPP_CONFIG}; then \
	    echo "Please run 'webapp-config -I -d ${LXR_PATH} lxr 0.3.1'"; \
	    echo "as root, then re-run 'make doc' or 'make doc_lxr'"; \
	    exit 1; \
	  else \
	    echo ssh ${LXR_SERVER} mkdir ${LXR_PATH}; \
	    ssh ${LXR_SERVER} mkdir ${LXR_PATH} || exit $$?; \
	    ssh ${LXR_SERVER} 'cd ${LXR_PATH} && for i in ${LXR_BINDIR}/* ${LXR_LIBDIR}; do echo ln -s $$i; done'; \
	    ssh ${LXR_SERVER} 'cd ${LXR_PATH} && for i in ${LXR_BINDIR}/* ${LXR_LIBDIR}; do echo ln -s $$i; ln -s $$i || exit $$?; done' || exit $$?; \
	  fi; \
	fi
	@ - for i in ${LXR_SOURCEROOT} ${LXR_INCPREFIX} ${LXR_DBDIR}; do \
	  echo ssh ${LXR_SERVER} mkdir ${LXR_PATH}/$$i; \
	  ssh ${LXR_SERVER} mkdir ${LXR_PATH}/$$i; \
	done
	scp lxr.conf ${LXR_SERVER}:${LXR_PATH}/ || exit $$?
	rsync -av -e ssh --delete --exclude CVS./ ${LXR_SERVER}:${LXR_PATH}/${LXR_SOURCEROOT}/ || exit $$?
	ssh ${LXR_SERVER} 'cd ${LXR_PATH}/${LXR_DBDIR} && ${LXR_GENXREF} ../${LXR_SOURCEROOT} && chmod 755 *' || exit $$?
	- ssh ${LXR_SERVER} ${LXR_GLIMPSEINDEX} -H ${LXR_PATH}/${LXR_DBDIR} ${LXR_PATH}/${LXR_SOURECROOT}
	ssh ${LXR_SERVER} chmod -R 755 ${LXR_PATH} || exit $$?

## Write LXR config file
lxr.conf:
	@ echo '# Define the base url for the LXR files.' >lxr.conf
	@ echo "baseurl: ${LXR_BASEURL}" >>lxr.conf
	@ echo >>lxr.conf
	@ echo '# These are the templates for the HTML heading, directory ' >>lxr.conf
	@ echo '# listing and footer, respectively.' >>lxr.conf
	@ echo "htmlhead: ${LXR_HTMLDIR}/template-head" >>lxr.conf
	@ echo "htmltail: ${LXR_HTMLDIR}/template-tail" >>lxr.conf
	@ echo "htmldir:  ${LXR_HTMLDIR}/template-dir" >>lxr.conf
	@ echo >>lxr.conf
	@ echo '# The source is here.' >>lxr.conf
	@ echo "sourceroot:  ${LXR_SOURCEROOT}" >>lxr.conf
	@ echo "srcrootname: ${LXR_SRCROOTNAME}" >>lxr.conf
	@ echo >>lxr.conf
	@ echo '# "#include <foo.h>" is mapped to this directory' >>lxr.conf
	@ echo '#  (in the LXR source tree)' >>lxr.conf
	@ echo "incprefix: ${LXR_INCPREFIX}" >>lxr.conf
	@ echo >>lxr.conf
	@ echo '# The database files go here.' >>lxr.conf
	@ echo "dbdir: ${LXR_DBDIR}" >>lxr.conf
	@ echo >>lxr.conf
	@ echo '# Glimpse can be found here.' >>lxr.conf
	@ if test -n "${LXR_GLIMPSEBIN}"; then \
	  echo "glimpsebin: ${LXR_GLIMPSEBIN}" >>lxr.conf; \
	else \
	  echo "#glimpsebin: ${LXR_GLIMPSEBIN}" >>lxr.conf; \
	fi

## Installs the ebuild into the $(PORTDIR_OVERLAY) directory
ebuild :
	@ if test -z "$(PORTDIR_OVERLAY)"; then\
	  echo;\
	  if ! test -e /etc/make.conf; then\
	    echo 'Cannot find /etc/make.conf, this is probably not a '\
	         'Gentoo system';\
	  else\
	    echo 'Cannot find PORTDIR_OVERLAY in /etc/make.conf; you '\
	         'must set this.';\
	    echo 'See http://www.gentoo.org/doc/en/gentoo-howto.xml#doc_chap6_sect2';\
	  fi;\
	  echo;\
	  exit 1;\
	fi
	@ echo cp distfiles/*.ebuild $(PORTDIR_OVERLAY)/$(PORTCAT)/${NAME}/
	@ cp distfiles/*.ebuild $(PORTDIR_OVERLAY)/$(PORTCAT)/${NAME}/
	- rm /usr/portage/distfiles/${NAME}-*
	@ for i in $(PORTDIR_OVERLAY)/$(PORTCAT)/${NAME}/*.ebuild; do\
	  $(EBUILD) $$i digest;\
	done

## Extracts the version information from the header files
gen_version :
	@ first=1;\
	for i in\
	`$(GREP) ${NAME}_VERSION $(PREFIX)/include/$(HFILES) 2>/dev/null | $(AWK) '{ print $$3 }'`;\
	do\
	  if test $$first -eq 1; then\
	    echo $$i >VERSION;\
	  else\
	    if test "$$i" != "$(VERSION)"; then\
	      echo \
	      '${NAME}_VERSION should be the same in *all* header files!';\
	      exit 1;\
	    fi\
	  fi;\
	done
	@ echo 'Version is: '$(VERSION)

help :
	@ echo 'Usage: make {<target>}'
	@ echo
	@ echo '  where valid <target>s are:'
	@ echo
	@ echo '    build:     builds the shared library'
	@ echo '    clean:     removes all object files'
	@ echo '    help:      displays this message'
	@ echo '    install:   installs to $(PREFIX)'
	@ echo '    test:      builds and runs test program(s)'
	@ echo '    uninstall: removes from $(PREFIX)'
	@ echo '    query:     displays the versions of ${NAME}'
	@ echo

install : build
	@ base=$(DESTDIR)$(PREFIX);\
	for i in $$base/lib $$base/include $$base/$(GNU_MAN)/$(MANDIR);\
	do\
	  test -d $$i
	  if $? -ne 0; then\
	    echo mkdir -p $$i;\
	    mkdir -p $$i;\
	  fi;\
	done
	cp $(SOFILES) $(DESTDIR)$(PREFIX)/lib/
	cp $(HFILES) $(DESTDIR)$(PREFIX)/include/
	cp $(MANFILE) $(DESTDIR)$(PREFIX)/$(GNU_MAN)/$(MANPAGE)
	${MAKE} ldconfig

## Run ldconfig if it is in the path
ldconfig :
	- /bin/env ldconfig

## Generate man pages
man :
	$(DOXYGEN)

query :
	@ src_ver=$(VERSION);\
	lib_ver=`$(GREP) ${NAME}_VERSION $(PREFIX)/include/$(HFILES) 2>/dev/null | $(AWK) '{ print $$3 }' `; \
	echo "Source version:    $$src_ver"; \
	if [ -n "$$lib_ver" ]; then \
	  echo "Installed version: $$lib_ver"; \
	fi

test :
	${MAKE} test-${NAME} "CFLAGS=${CFLAGS} -g"

uninstall :
	- rm $(PREFIX)/lib/$(SOFILES)
	- rm $(PREFIX)/include/$(HFILES)
	- rm $(PREFIX)/$(GNU_MAN)/$(MANPAGE)

## Use valgrind on the test program
valgrind: test
	@ if ! test -x ${VALGRIND_BIN}; then \
	  echo 1>&2; \
	  echo '${VALGRIND_BIN} does not exist or is not executable!' 1>&2; \
	  echo 'Try setting the VALGRIND_BIN macro:' 1>&2; \
	  echo 'make valgrind VALGRIND_BIN=/usr/local/bin/valgrind' 1>&2; \
	  echo 1>&2; \
	  exit 1; \
	fi
	${VALGRIND_BIN} ${VALGRIND_OPTS} --logfile=valgrind-log_test-${NAME} ./test-__PROJECT__

test-__PROJECT__ : __PROJECT__.c __PROJECT__.h test-__PROJECT__.c
	${CC} ${CFLAGS} -o test-__PROJECT__ __PROJECT__.c test-__PROJECT__.c
